==> selenium-api-2.48.2-sources.jar: org/openqa/selenium/interactions/internal/BaseAction.java <==
// Licensed to the Software Freedom Conservancy (SFC) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The SFC licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package org.openqa.selenium.interactions.internal;

import org.openqa.selenium.WebElement;
import org.openqa.selenium.internal.HasIdentity;
import org.openqa.selenium.internal.Locatable;
import org.openqa.selenium.internal.WrapsElement;

/**
 * Base class for all actions.
 */
public abstract class BaseAction {
  protected final Locatable where;

  /**
   * Common c'tor - a locatable element is provided.
   *
   * @param actionLocation provider of coordinates for the action.
   */
  protected BaseAction(Locatable actionLocation) {
    this.where = actionLocation;
  }

  /**
   * No locatable element provided - action in the context of the previous action.
   */
  protected BaseAction() {
    this.where = null;
  }

  protected String getTargetId() {
    if (!(where instanceof WebElement)) {
      return null;
    }

    WebElement target = (WebElement) where;

    while (target instanceof WrapsElement) {
      target = ((WrapsElement) target).getWrappedElement();
    }

    if (target instanceof HasIdentity) {
      return ((HasIdentity) target).getId();
    }

    return null;
  }
}

==> selenium-api-3.14.0-sources.jar: org/openqa/selenium/interactions/internal/BaseAction.java <==
// Licensed to the Software Freedom Conservancy (SFC) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The SFC licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package org.openqa.selenium.interactions.internal;

import org.openqa.selenium.WebElement;

import java.util.Optional;

/**
 * Base class for all actions.
 */
public abstract class BaseAction {
  protected final Locatable where;

  /**
   * Common c'tor - a locatable element is provided.
   *
   * @param actionLocation provider of coordinates for the action.
   */
  protected BaseAction(Locatable actionLocation) {
    this.where = actionLocation;
  }

  protected Optional<WebElement> getTargetElement() {
    if (where == null) {
      return Optional.empty();
    }

    if (!(where.getCoordinates().getAuxiliary() instanceof WebElement)) {
      throw new IllegalStateException(String.format(
          "%s: Unable to find element to use: %s",
          this,
          where.getCoordinates()));
    }

    return Optional.of((WebElement) where.getCoordinates().getAuxiliary());
  }
}
